@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2012 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
  *
  * Andreas Langer <an.langer@gmx.de>, Marek Lindner <lindner_marek@yahoo.de>
  *
@@ -20,11 +20,20 @@
  */
 
 #ifndef SOURCE_VERSION
-#define SOURCE_VERSION "2012.1.0"
+#define SOURCE_VERSION "2013.3.0"
 #endif
 
 #define SOCKET_PATH_FMT "%s/batman_adv/%s/socket"
 
 #define EXIT_NOSUCCESS 2
 
+#define OPT_LONG_MAX_LEN 25
+#define OPT_SHORT_MAX_LEN 5
+
+#define DEBUG_TABLE_PATH_MAX_LEN 20
+#define SETTINGS_PATH_MAX_LEN 25
+
 #define __packed __attribute((packed))   /* linux kernel compat */
+#define BIT(nr)                 (1UL << (nr)) /* linux kernel compat */
+
+extern char module_ver_path[];
