@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2012 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2009-2013 B.A.T.M.A.N. contributors:
  *
  * Marek Lindner <lindner_marek@yahoo.de>
  *
@@ -49,11 +49,69 @@ const char *sysfs_param_server[] = {
 	NULL,
 };
 
+const struct settings_data batctl_settings[BATCTL_SETTINGS_NUM] = {
+	{
+		.opt_long = "orig_interval",
+		.opt_short = "it",
+		.sysfs_name = "orig_interval",
+		.params = NULL,
+	},
+	{
+		.opt_long = "ap_isolation",
+		.opt_short = "ap",
+		.sysfs_name = "ap_isolation",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "bridge_loop_avoidance",
+		.opt_short = "bl",
+		.sysfs_name = "bridge_loop_avoidance",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "distributed_arp_table",
+		.opt_short = "dat",
+		.sysfs_name = "distributed_arp_table",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "vis_mode",
+		.opt_short = "vm",
+		.sysfs_name = "vis_mode",
+		.params = sysfs_param_server,
+	},
+	{
+		.opt_long = "aggregation",
+		.opt_short = "ag",
+		.sysfs_name = "aggregated_ogms",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "bonding",
+		.opt_short = "b",
+		.sysfs_name = "bonding",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "fragmentation",
+		.opt_short = "f",
+		.sysfs_name = "fragmentation",
+		.params = sysfs_param_enable,
+	},
+	{
+		.opt_long = "network_coding",
+		.opt_short = "nc",
+		.sysfs_name = "network_coding",
+		.params = sysfs_param_enable,
+	}
+
+};
+
 static void interface_usage(void)
 {
-	printf("Usage: batctl interface [options] [add|del iface(s)] \n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
+	fprintf(stderr, "Usage: batctl [options] interface [parameters] [add|del iface(s)]\n");
+	fprintf(stderr, "parameters:\n");
+	fprintf(stderr, " \t -h print this help\n");
 }
 
 static int print_interfaces(char *mesh_iface)
@@ -63,23 +121,29 @@ static int print_interfaces(char *mesh_iface)
 	char *path_buff;
 	int res;
 
+	if (!file_exists(module_ver_path)) {
+		fprintf(stderr, "Error - batman-adv module has not been loaded\n");
+		goto err;
+	}
+
 	path_buff = malloc(PATH_BUFF_LEN);
 	if (!path_buff) {
-		printf("Error - could not allocate path buffer: out of memory ?\n");
+		fprintf(stderr, "Error - could not allocate path buffer: out of memory ?\n");
 		goto err;
 	}
 
 	iface_base_dir = opendir(SYS_IFACE_PATH);
 	if (!iface_base_dir) {
-		printf("Error - the directory '%s' could not be read: %s\n",
+		fprintf(stderr, "Error - the directory '%s' could not be read: %s\n",
 		       SYS_IFACE_PATH, strerror(errno));
-		printf("Is the batman-adv module loaded and sysfs mounted ?\n");
+		fprintf(stderr, "Is the batman-adv module loaded and sysfs mounted ?\n");
 		goto err_buff;
 	}
 
 	while ((iface_dir = readdir(iface_base_dir)) != NULL) {
 		snprintf(path_buff, PATH_BUFF_LEN, SYS_MESH_IFACE_FMT, iface_dir->d_name);
-		res = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0);
+		path_buff[PATH_BUFF_LEN - 1] = '\0';
+		res = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0, 0);
 		if (res != EXIT_SUCCESS)
 			continue;
 
@@ -96,9 +160,10 @@ static int print_interfaces(char *mesh_iface)
 		line_ptr = NULL;
 
 		snprintf(path_buff, PATH_BUFF_LEN, SYS_IFACE_STATUS_FMT, iface_dir->d_name);
-		res = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0);
+		path_buff[PATH_BUFF_LEN - 1] = '\0';
+		res = read_file("", path_buff, USE_READ_BUFF | SILENCE_ERRORS, 0, 0, 0);
 		if (res != EXIT_SUCCESS) {
-			printf("<error reading status>\n");
+			fprintf(stderr, "<error reading status>\n");
 			continue;
 		}
 
@@ -140,19 +205,44 @@ int interface(char *mesh_iface, int argc, char **argv)
 
 	if ((strcmp(argv[1], "add") != 0) && (strcmp(argv[1], "a") != 0) &&
 	    (strcmp(argv[1], "del") != 0) && (strcmp(argv[1], "d") != 0)) {
-		printf("Error - unknown argument specified: %s\n", argv[1]);
+		fprintf(stderr, "Error - unknown argument specified: %s\n", argv[1]);
+		interface_usage();
+		goto err;
+	}
+
+	if (argc == 2) {
+		fprintf(stderr, "Error - missing interface name(s) after '%s'\n", argv[1]);
 		interface_usage();
 		goto err;
 	}
 
 	path_buff = malloc(PATH_BUFF_LEN);
 	if (!path_buff) {
-		printf("Error - could not allocate path buffer: out of memory ?\n");
+		fprintf(stderr, "Error - could not allocate path buffer: out of memory ?\n");
 		goto err;
 	}
 
 	for (i = 2; i < argc; i++) {
 		snprintf(path_buff, PATH_BUFF_LEN, SYS_MESH_IFACE_FMT, argv[i]);
+		path_buff[PATH_BUFF_LEN - 1] = '\0';
+
+		if (!file_exists(path_buff)) {
+			snprintf(path_buff, PATH_BUFF_LEN, SYS_IFACE_DIR, argv[i]);
+			path_buff[PATH_BUFF_LEN - 1] = '\0';
+
+			if (!file_exists(path_buff)) {
+				fprintf(stderr, "Error - interface does not exist: %s\n", argv[i]);
+				continue;
+			}
+
+			if (!file_exists(module_ver_path)) {
+				fprintf(stderr, "Error - batman-adv module has not been loaded\n");
+				goto err;
+			}
+
+			fprintf(stderr, "Error - interface type not supported by batman-adv: %s\n", argv[i]);
+			continue;
+		}
 
 		if (argv[1][0] == 'a')
 			res = write_file("", path_buff, mesh_iface, NULL);
@@ -174,15 +264,18 @@ err:
 
 static void log_level_usage(void)
 {
-	printf("Usage: batctl [options] loglevel [level[ level[ level]]...]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-	printf("levels:\n");
-	printf(" \t none    Debug logging is disabled\n");
-	printf(" \t all     Print messages from all below\n");
-	printf(" \t batman  Messages related to routing / flooding / broadcasting\n");
-	printf(" \t routes  Messages related to route added / changed / deleted\n");
-	printf(" \t tt      Messages related to translation table operations\n");
+	fprintf(stderr, "Usage: batctl [options] loglevel [parameters] [level[ level[ level]]...]\n");
+	fprintf(stderr, "parameters:\n");
+	fprintf(stderr, " \t -h print this help\n");
+	fprintf(stderr, "levels:\n");
+	fprintf(stderr, " \t none    Debug logging is disabled\n");
+	fprintf(stderr, " \t all     Print messages from all below\n");
+	fprintf(stderr, " \t batman  Messages related to routing / flooding / broadcasting\n");
+	fprintf(stderr, " \t routes  Messages related to route added / changed / deleted\n");
+	fprintf(stderr, " \t tt      Messages related to translation table operations\n");
+	fprintf(stderr, " \t bla     Messages related to bridge loop avoidance\n");
+	fprintf(stderr, " \t dat     Messages related to arp snooping and distributed arp table\n");
+	fprintf(stderr, " \t nc      Messages related to network coding\n");
 }
 
 int handle_loglevel(char *mesh_iface, int argc, char **argv)
@@ -206,6 +299,7 @@ int handle_loglevel(char *mesh_iface, int argc, char **argv)
 
 	path_buff = malloc(PATH_BUFF_LEN);
 	snprintf(path_buff, PATH_BUFF_LEN, SYS_BATIF_PATH_FMT, mesh_iface);
+	path_buff[PATH_BUFF_LEN - 1] = '\0';
 
 	if (argc != 1) {
 		for (i = 1; i < argc; i++) {
@@ -213,14 +307,20 @@ int handle_loglevel(char *mesh_iface, int argc, char **argv)
 				log_level = 0;
 				break;
 			} else if (strcmp(argv[i], "all") == 0) {
-				log_level = 7;
+				log_level = 63;
 				break;
 			} else if (strcmp(argv[i], "batman") == 0)
-				log_level |= (1 << 0);
+				log_level |= BIT(0);
 			else if (strcmp(argv[i], "routes") == 0)
-				log_level |= (1 << 1);
+				log_level |= BIT(1);
 			else if (strcmp(argv[i], "tt") == 0)
-				log_level |= (1 << 2);
+				log_level |= BIT(2);
+			else if (strcmp(argv[i], "bla") == 0)
+				log_level |= BIT(3);
+			else if (strcmp(argv[i], "dat") == 0)
+				log_level |= BIT(4);
+			else if (strcmp(argv[i], "nc") == 0)
+				log_level |= BIT(5);
 			else {
 				log_level_usage();
 				goto out;
@@ -228,11 +328,12 @@ int handle_loglevel(char *mesh_iface, int argc, char **argv)
 		}
 
 		snprintf(str, sizeof(str), "%i", log_level);
+		str[sizeof(str) - 1] = '\0';
 		res = write_file(path_buff, SYS_LOG_LEVEL, str, NULL);
 		goto out;
 	}
 
-	res = read_file(path_buff, SYS_LOG_LEVEL, USE_READ_BUFF, 0, 0);
+	res = read_file(path_buff, SYS_LOG_LEVEL, USE_READ_BUFF, 0, 0, 0);
 
 	if (res != EXIT_SUCCESS)
 		goto out;
@@ -241,74 +342,42 @@ int handle_loglevel(char *mesh_iface, int argc, char **argv)
 
 	printf("[%c] %s (%s)\n", (!log_level) ? 'x' : ' ',
 	       "all debug output disabled", "none");
-	printf("[%c] %s (%s)\n", (log_level & 1) ? 'x' : ' ',
+	printf("[%c] %s (%s)\n", (log_level & BIT(0)) ? 'x' : ' ',
 	       "messages related to routing / flooding / broadcasting",
 	       "batman");
-	printf("[%c] %s (%s)\n", (log_level & 2) ? 'x' : ' ',
+	printf("[%c] %s (%s)\n", (log_level & BIT(1)) ? 'x' : ' ',
 	       "messages related to route added / changed / deleted", "routes");
-	printf("[%c] %s (%s)\n", (log_level & 4) ? 'x' : ' ',
+	printf("[%c] %s (%s)\n", (log_level & BIT(2)) ? 'x' : ' ',
 	       "messages related to translation table operations", "tt");
+	printf("[%c] %s (%s)\n", (log_level & BIT(3)) ? 'x' : ' ',
+	       "messages related to bridge loop avoidance", "bla");
+	printf("[%c] %s (%s)\n", (log_level & BIT(4)) ? 'x' : ' ',
+	       "messages related to arp snooping and distributed arp table", "dat");
+	printf("[%c] %s (%s)\n", (log_level & BIT(5)) ? 'x' : ' ',
+	       "messages related to network coding", "nc");
 
 out:
-	if (errno == ENOENT)
-		printf("To increase the log level you need to compile the module with debugging enabled (see the README)\n");
-
 	free(path_buff);
 	return res;
 }
 
-void aggregation_usage(void)
+static void settings_usage(int setting)
 {
-	printf("Usage: batctl [options] aggregation [0|1]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
+	fprintf(stderr, "Usage: batctl [options] %s|%s [parameters]",
+	       (char *)batctl_settings[setting].opt_long, (char *)batctl_settings[setting].opt_short);
+
+	if (batctl_settings[setting].params == sysfs_param_enable)
+		fprintf(stderr, " [0|1]\n");
+	else if (batctl_settings[setting].params == sysfs_param_server)
+		fprintf(stderr, " [client|server]\n");
+	else
+		fprintf(stderr, "\n");
+
+	fprintf(stderr, "parameters:\n");
+	fprintf(stderr, " \t -h print this help\n");
 }
 
-void bonding_usage(void)
-{
-	printf("Usage: batctl [options] bonding [0|1]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-void gw_mode_usage(void)
-{
-	printf("Usage: batctl [options] gw_mode [mode] [sel_class|bandwidth]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-void vis_mode_usage(void)
-{
-	printf("Usage: batctl [options] vis_mode [mode]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-void orig_interval_usage(void)
-{
-	printf("Usage: batctl [options] interval \n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-void fragmentation_usage(void)
-{
-	printf("Usage: batctl [options] fragmentation [0|1]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-void ap_isolation_usage(void)
-{
-	printf("Usage: batctl [options] ap_isolation [0|1]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-}
-
-int handle_sys_setting(char *mesh_iface, int argc, char **argv,
-		       char *file_path, void setting_usage(void),
-		       const char *sysfs_param[])
+int handle_sys_setting(char *mesh_iface, int setting, int argc, char **argv)
 {
 	int optchar, res = EXIT_FAILURE;
 	char *path_buff;
@@ -317,26 +386,28 @@ int handle_sys_setting(char *mesh_iface, int argc, char **argv,
 	while ((optchar = getopt(argc, argv, "h")) != -1) {
 		switch (optchar) {
 		case 'h':
-			setting_usage();
+			settings_usage(setting);
 			return EXIT_SUCCESS;
 		default:
-			setting_usage();
+			settings_usage(setting);
 			return EXIT_FAILURE;
 		}
 	}
 
 	path_buff = malloc(PATH_BUFF_LEN);
 	snprintf(path_buff, PATH_BUFF_LEN, SYS_BATIF_PATH_FMT, mesh_iface);
+	path_buff[PATH_BUFF_LEN - 1] = '\0';
 
 	if (argc == 1) {
-		res = read_file(path_buff, file_path, NO_FLAGS, 0, 0);
+		res = read_file(path_buff, (char *)batctl_settings[setting].sysfs_name,
+				NO_FLAGS, 0, 0, 0);
 		goto out;
 	}
 
-	if (!sysfs_param)
+	if (!batctl_settings[setting].params)
 		goto write_file;
 
-	ptr = sysfs_param;
+	ptr = batctl_settings[setting].params;
 	while (*ptr) {
 		if (strcmp(*ptr, argv[1]) == 0)
 			goto write_file;
@@ -344,25 +415,33 @@ int handle_sys_setting(char *mesh_iface, int argc, char **argv,
 		ptr++;
 	}
 
-	printf("Error - the supplied argument is invalid: %s\n", argv[1]);
-	printf("The following values are allowed:\n");
+	fprintf(stderr, "Error - the supplied argument is invalid: %s\n", argv[1]);
+	fprintf(stderr, "The following values are allowed:\n");
 
-	ptr = sysfs_param;
+	ptr = batctl_settings[setting].params;
 	while (*ptr) {
-		printf(" * %s\n", *ptr);
+		fprintf(stderr, " * %s\n", *ptr);
 		ptr++;
 	}
 
 	goto out;
 
 write_file:
-	res = write_file(path_buff, file_path, argv[1], argc > 2 ? argv[2] : NULL);
+	res = write_file(path_buff, (char *)batctl_settings[setting].sysfs_name,
+			 argv[1], argc > 2 ? argv[2] : NULL);
 
 out:
 	free(path_buff);
 	return res;
 }
 
+static void gw_mode_usage(void)
+{
+	fprintf(stderr, "Usage: batctl [options] gw_mode [mode] [sel_class|bandwidth]\n");
+	fprintf(stderr, "options:\n");
+	fprintf(stderr, " \t -h print this help\n");
+}
+
 int handle_gw_setting(char *mesh_iface, int argc, char **argv)
 {
 	int optchar, res = EXIT_FAILURE;
@@ -382,9 +461,10 @@ int handle_gw_setting(char *mesh_iface, int argc, char **argv)
 
 	path_buff = malloc(PATH_BUFF_LEN);
 	snprintf(path_buff, PATH_BUFF_LEN, SYS_BATIF_PATH_FMT, mesh_iface);
+	path_buff[PATH_BUFF_LEN - 1] = '\0';
 
 	if (argc == 1) {
-		res = read_file(path_buff, SYS_GW_MODE, USE_READ_BUFF, 0, 0);
+		res = read_file(path_buff, SYS_GW_MODE, USE_READ_BUFF, 0, 0, 0);
 
 		if (res != EXIT_SUCCESS)
 			goto out;
@@ -404,10 +484,10 @@ int handle_gw_setting(char *mesh_iface, int argc, char **argv)
 
 		switch (gw_mode) {
 		case GW_MODE_CLIENT:
-			res = read_file(path_buff, SYS_GW_SEL, USE_READ_BUFF, 0, 0);
+			res = read_file(path_buff, SYS_GW_SEL, USE_READ_BUFF, 0, 0, 0);
 			break;
 		case GW_MODE_SERVER:
-			res = read_file(path_buff, SYS_GW_BW, USE_READ_BUFF, 0, 0);
+			res = read_file(path_buff, SYS_GW_BW, USE_READ_BUFF, 0, 0, 0);
 			break;
 		default:
 			printf("off\n");
@@ -464,12 +544,12 @@ int handle_gw_setting(char *mesh_iface, int argc, char **argv)
 	goto out;
 
 opt_err:
-	printf("Error - the supplied argument is invalid: %s\n", argv[1]);
-	printf("The following values are allowed:\n");
+	fprintf(stderr, "Error - the supplied argument is invalid: %s\n", argv[1]);
+	fprintf(stderr, "The following values are allowed:\n");
 
 	ptr = sysfs_param_server;
 	while (*ptr) {
-		printf(" * %s\n", *ptr);
+		fprintf(stderr, " * %s\n", *ptr);
 		ptr++;
 	}
 
