@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2012 B.A.T.M.A.N. contributors:
+ * Copyright (C) 2007-2013 B.A.T.M.A.N. contributors:
  *
  * Andreas Langer <an.langer@gmx.de>
  *
@@ -43,10 +43,13 @@
 #include "bat-hosts.h"
 #include "functions.h"
 
+#ifndef ETH_P_BATMAN
+#define ETH_P_BATMAN	0x4305
+#endif /* ETH_P_BATMAN */
 
 #define LEN_CHECK(buff_len, check_len, desc) \
 if ((size_t)(buff_len) < (check_len)) { \
-	printf("Warning - dropping received %s packet as it is smaller than expected (%zu): %zu\n", \
+	fprintf(stderr, "Warning - dropping received %s packet as it is smaller than expected (%zu): %zu\n", \
 		desc, (check_len), (size_t)(buff_len)); \
 	return; \
 }
@@ -59,22 +62,23 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 
 static void tcpdump_usage(void)
 {
-	printf("Usage: batctl tcpdump [options] interface [interface]\n");
-	printf("options:\n");
-	printf(" \t -h print this help\n");
-	printf(" \t -n don't convert addresses to bat-host names\n");
-	printf(" \t -p dump specific packet type\n");
-	printf(" \t -x dump all packet types except specified\n");
-	printf("packet types:\n");
-	printf(" \t\t%3d - batman ogm packets\n", DUMP_TYPE_BATOGM);
-	printf(" \t\t%3d - batman icmp packets\n", DUMP_TYPE_BATICMP);
-	printf(" \t\t%3d - batman unicast packets\n", DUMP_TYPE_BATUCAST);
-	printf(" \t\t%3d - batman broadcast packets\n", DUMP_TYPE_BATBCAST);
-	printf(" \t\t%3d - batman vis packets\n", DUMP_TYPE_BATVIS);
-	printf(" \t\t%3d - batman fragmented packets\n", DUMP_TYPE_BATFRAG);
-	printf(" \t\t%3d - batman tt / roaming packets\n", DUMP_TYPE_BATTT);
-	printf(" \t\t%3d - non batman packets\n", DUMP_TYPE_NONBAT);
-	printf(" \t\t%3d - batman ogm & non batman packets\n", DUMP_TYPE_BATOGM | DUMP_TYPE_NONBAT);
+	fprintf(stderr, "Usage: batctl tcpdump [parameters] interface [interface]\n");
+	fprintf(stderr, "parameters:\n");
+	fprintf(stderr, " \t -c compat filter - only display packets matching own compat version (%i)\n", BATADV_COMPAT_VERSION);
+	fprintf(stderr, " \t -h print this help\n");
+	fprintf(stderr, " \t -n don't convert addresses to bat-host names\n");
+	fprintf(stderr, " \t -p dump specific packet type\n");
+	fprintf(stderr, " \t -x dump all packet types except specified\n");
+	fprintf(stderr, "packet types:\n");
+	fprintf(stderr, " \t\t%3d - batman ogm packets\n", DUMP_TYPE_BATOGM);
+	fprintf(stderr, " \t\t%3d - batman icmp packets\n", DUMP_TYPE_BATICMP);
+	fprintf(stderr, " \t\t%3d - batman unicast packets\n", DUMP_TYPE_BATUCAST);
+	fprintf(stderr, " \t\t%3d - batman broadcast packets\n", DUMP_TYPE_BATBCAST);
+	fprintf(stderr, " \t\t%3d - batman vis packets\n", DUMP_TYPE_BATVIS);
+	fprintf(stderr, " \t\t%3d - batman fragmented packets\n", DUMP_TYPE_BATFRAG);
+	fprintf(stderr, " \t\t%3d - batman tt / roaming packets\n", DUMP_TYPE_BATTT);
+	fprintf(stderr, " \t\t%3d - non batman packets\n", DUMP_TYPE_NONBAT);
+	fprintf(stderr, " \t\t%3d - batman ogm & non batman packets\n", DUMP_TYPE_BATOGM | DUMP_TYPE_NONBAT);
 }
 
 static int print_time(void)
@@ -89,9 +93,75 @@ static int print_time(void)
 	return 1;
 }
 
-static void dump_arp(unsigned char *packet_buff, ssize_t buff_len, int time_printed)
+static int dump_bla2_claim(struct ether_header *eth_hdr,
+			   struct ether_arp *arphdr, int read_opt)
+{
+	uint8_t bla_claim_magic[3] = {0xff, 0x43, 0x05};
+	struct batadv_bla_claim_dst *bla_dst;
+	int arp_is_bla2_claim = 0;
+	uint8_t *hw_src, *hw_dst;
+
+	if (arphdr->ea_hdr.ar_hrd != htons(ARPHRD_ETHER))
+		goto out;
+
+	if (arphdr->ea_hdr.ar_pro != htons(ETH_P_IP))
+		goto out;
+
+	if (arphdr->ea_hdr.ar_hln != ETH_ALEN)
+		goto out;
+
+	if (arphdr->ea_hdr.ar_pln != 4)
+		goto out;
+
+	hw_src = arphdr->arp_sha;
+	hw_dst = arphdr->arp_tha;
+	bla_dst = (struct batadv_bla_claim_dst *)hw_dst;
+
+	if (memcmp(bla_dst->magic, bla_claim_magic, sizeof(bla_claim_magic)) != 0)
+		goto out;
+
+	switch (bla_dst->type) {
+	case BATADV_CLAIM_TYPE_CLAIM:
+		printf("BLA CLAIM, backbone %s, ",
+		       get_name_by_macaddr((struct ether_addr *)hw_src, read_opt));
+		printf("client %s, bla group %04x\n",
+		       get_name_by_macaddr((struct ether_addr *)eth_hdr->ether_shost, read_opt),
+		       ntohs(bla_dst->group));
+		break;
+	case BATADV_CLAIM_TYPE_UNCLAIM:
+		printf("BLA UNCLAIM, backbone %s, ",
+		       get_name_by_macaddr((struct ether_addr *)eth_hdr->ether_shost, read_opt));
+		printf("client %s, bla group %04x\n",
+		       get_name_by_macaddr((struct ether_addr *)hw_src, read_opt),
+		       ntohs(bla_dst->group));
+		break;
+	case BATADV_CLAIM_TYPE_ANNOUNCE:
+		printf("BLA ANNOUNCE, backbone %s, bla group %04x, crc %04x\n",
+		       get_name_by_macaddr((struct ether_addr *)eth_hdr->ether_shost, read_opt),
+		       ntohs(bla_dst->group), ntohs(*((uint16_t *)(&hw_src[4]))));
+		break;
+	case BATADV_CLAIM_TYPE_REQUEST:
+		printf("BLA REQUEST, src backbone %s, ",
+		       get_name_by_macaddr((struct ether_addr *)hw_src, read_opt));
+		printf("dst backbone %s\n",
+		       get_name_by_macaddr((struct ether_addr *)eth_hdr->ether_dhost, read_opt));
+		break;
+	default:
+		printf("BLA UNKNOWN, type %hhu\n", bla_dst->type);
+		break;
+	}
+
+	arp_is_bla2_claim = 1;
+
+out:
+	return arp_is_bla2_claim;
+}
+
+static void dump_arp(unsigned char *packet_buff, ssize_t buff_len,
+		     struct ether_header *eth_hdr, int read_opt, int time_printed)
 {
 	struct ether_arp *arphdr;
+	int arp_is_bla2_claim;
 
 	LEN_CHECK((size_t)buff_len, sizeof(struct ether_arp), "ARP");
 
@@ -107,6 +177,10 @@ static void dump_arp(unsigned char *packet_buff, ssize_t buff_len, int time_prin
 			ether_ntoa_long((struct ether_addr *)&arphdr->arp_sha), buff_len);
 		break;
 	case ARPOP_REPLY:
+		arp_is_bla2_claim = dump_bla2_claim(eth_hdr, arphdr, read_opt);
+		if (arp_is_bla2_claim)
+			break;
+
 		printf("ARP, Reply %s is-at %s, length %zd\n", inet_ntoa(*(struct in_addr *)&arphdr->arp_spa),
 			ether_ntoa_long((struct ether_addr *)&arphdr->arp_sha), buff_len);
 		break;
@@ -253,23 +327,23 @@ static void dump_vlan(unsigned char *packet_buff, ssize_t buff_len, int read_opt
 
 static void dump_batman_tt(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct tt_query_packet *tt_query_packet;
+	struct batadv_tt_query_packet *tt_query_packet;
 	char *tt_desc, *tt_data, tt_type;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct tt_query_packet), "BAT TT");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_tt_query_packet), "BAT TT");
 
-	tt_query_packet = (struct tt_query_packet *)(packet_buff + sizeof(struct ether_header));
+	tt_query_packet = (struct batadv_tt_query_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		print_time();
 
-	switch (tt_query_packet->flags & TT_QUERY_TYPE_MASK) {
-	case TT_REQUEST:
+	switch (tt_query_packet->flags & BATADV_TT_QUERY_TYPE_MASK) {
+	case BATADV_TT_REQUEST:
 		tt_desc = "request";
 		tt_data = "crc";
 		tt_type = 'Q';
 		break;
-	case TT_RESPONSE:
+	case BATADV_TT_RESPONSE:
 		tt_desc = "response";
 		tt_data = "entries";
 		tt_type = 'P';
@@ -288,17 +362,17 @@ static void dump_batman_tt(unsigned char *packet_buff, ssize_t buff_len, int rea
 	       get_name_by_macaddr((struct ether_addr *)tt_query_packet->dst, read_opt),
 	       tt_desc, tt_query_packet->ttvn, tt_data, ntohs(tt_query_packet->tt_data),
 	       tt_query_packet->header.ttl, tt_query_packet->header.version,
-	       tt_type, (tt_query_packet->flags & TT_FULL_TABLE ? 'F' : '.'),
+	       tt_type, (tt_query_packet->flags & BATADV_TT_FULL_TABLE ? 'F' : '.'),
 	       (size_t)buff_len - sizeof(struct ether_header));
 }
 
 static void dump_batman_roam(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct roam_adv_packet *roam_adv_packet;
+	struct batadv_roam_adv_packet *roam_adv_packet;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct roam_adv_packet), "BAT ROAM");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_roam_adv_packet), "BAT ROAM");
 
-	roam_adv_packet = (struct roam_adv_packet *)(packet_buff + sizeof(struct ether_header));
+	roam_adv_packet = (struct batadv_roam_adv_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		print_time();
@@ -315,15 +389,15 @@ static void dump_batman_roam(unsigned char *packet_buff, ssize_t buff_len, int r
 	       (size_t)buff_len - sizeof(struct ether_header));
 }
 
-static void dump_batman_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
+static void dump_batman_iv_ogm(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
-	struct batman_ogm_packet *batman_ogm_packet;
+	struct batadv_ogm_packet *batman_ogm_packet;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batman_ogm_packet), "BAT OGM");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_ogm_packet), "BAT IV OGM");
 
 	ether_header = (struct ether_header *)packet_buff;
-	batman_ogm_packet = (struct batman_ogm_packet *)(packet_buff + sizeof(struct ether_header));
+	batman_ogm_packet = (struct batadv_ogm_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		print_time();
@@ -331,25 +405,26 @@ static void dump_batman_ogm(unsigned char *packet_buff, ssize_t buff_len, int re
 	printf("BAT %s: ",
 	       get_name_by_macaddr((struct ether_addr *)batman_ogm_packet->orig, read_opt));
 
-	printf("OGM via neigh %s, seq %u, tq %3d, ttvn %d, ttcrc %hu, ttl %2d, v %d, flags [%c%c%c%c], length %zu\n",
+	printf("OGM IV via neigh %s, seq %u, tq %3d, ttvn %d, ttcrc %hu, ttl %2d, v %d, flags [%c%c%c%c%c], length %zu\n",
 	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt),
 	       ntohl(batman_ogm_packet->seqno), batman_ogm_packet->tq, batman_ogm_packet->ttvn,
 	       ntohs(batman_ogm_packet->tt_crc), batman_ogm_packet->header.ttl, batman_ogm_packet->header.version,
-	       (batman_ogm_packet->flags & DIRECTLINK ? 'D' : '.'),
-	       (batman_ogm_packet->flags & VIS_SERVER ? 'V' : '.'),
-	       (batman_ogm_packet->flags & PRIMARIES_FIRST_HOP ? 'F' : '.'),
+	       (batman_ogm_packet->flags & BATADV_NOT_BEST_NEXT_HOP ? 'N' : '.'),
+	       (batman_ogm_packet->flags & BATADV_DIRECTLINK ? 'D' : '.'),
+	       (batman_ogm_packet->flags & BATADV_VIS_SERVER ? 'V' : '.'),
+	       (batman_ogm_packet->flags & BATADV_PRIMARIES_FIRST_HOP ? 'F' : '.'),
 	       (batman_ogm_packet->gw_flags ? 'G' : '.'),
 	       (size_t)buff_len - sizeof(struct ether_header));
 }
 
 static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct icmp_packet *icmp_packet;
+	struct batadv_icmp_packet *icmp_packet;
 	char *name;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct icmp_packet), "BAT ICMP");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_icmp_packet), "BAT ICMP");
 
-	icmp_packet = (struct icmp_packet *)(packet_buff + sizeof(struct ether_header));
+	icmp_packet = (struct batadv_icmp_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		print_time();
@@ -359,19 +434,19 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 	name = get_name_by_macaddr((struct ether_addr *)icmp_packet->dst, read_opt);
 
 	switch (icmp_packet->msg_type) {
-	case ECHO_REPLY:
+	case BATADV_ECHO_REPLY:
 		printf("%s: ICMP echo reply, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
 			icmp_packet->header.ttl, icmp_packet->header.version,
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
-	case ECHO_REQUEST:
+	case BATADV_ECHO_REQUEST:
 		printf("%s: ICMP echo request, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
 			icmp_packet->header.ttl, icmp_packet->header.version,
 			(size_t)buff_len - sizeof(struct ether_header));
 		break;
-	case TTL_EXCEEDED:
+	case BATADV_TTL_EXCEEDED:
 		printf("%s: ICMP time exceeded in-transit, id %hhu, seq %hu, ttl %2d, v %d, length %zu\n",
 			name, icmp_packet->uid, ntohs(icmp_packet->seqno),
 			icmp_packet->header.ttl, icmp_packet->header.version,
@@ -387,14 +462,14 @@ static void dump_batman_icmp(unsigned char *packet_buff, ssize_t buff_len, int r
 static void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
-	struct unicast_packet *unicast_packet;
+	struct batadv_unicast_packet *unicast_packet;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct unicast_packet), "BAT UCAST");
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header) - sizeof(struct unicast_packet),
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_unicast_packet), "BAT UCAST");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header) - sizeof(struct batadv_unicast_packet),
 		sizeof(struct ether_header), "BAT UCAST (unpacked)");
 
 	ether_header = (struct ether_header *)packet_buff;
-	unicast_packet = (struct unicast_packet *)(packet_buff + sizeof(struct ether_header));
+	unicast_packet = (struct batadv_unicast_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		time_printed = print_time();
@@ -406,22 +481,22 @@ static void dump_batman_ucast(unsigned char *packet_buff, ssize_t buff_len, int
 	       get_name_by_macaddr((struct ether_addr *)unicast_packet->dest, read_opt),
 	       unicast_packet->ttvn, unicast_packet->header.ttl);
 
-	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct unicast_packet),
-		      buff_len - ETH_HLEN - sizeof(struct unicast_packet),
+	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_packet),
+		      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_packet),
 		      read_opt, time_printed);
 }
 
 static void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
 	struct ether_header *ether_header;
-	struct bcast_packet *bcast_packet;
+	struct batadv_bcast_packet *bcast_packet;
 
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct bcast_packet), "BAT BCAST");
-	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header) - sizeof(struct bcast_packet),
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_bcast_packet), "BAT BCAST");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header) - sizeof(struct batadv_bcast_packet),
 	          sizeof(struct ether_header), "BAT BCAST (unpacked)");
 
 	ether_header = (struct ether_header *)packet_buff;
-	bcast_packet = (struct bcast_packet *)(packet_buff + sizeof(struct ether_header));
+	bcast_packet = (struct batadv_bcast_packet *)(packet_buff + sizeof(struct ether_header));
 
 	if (!time_printed)
 		time_printed = print_time();
@@ -433,19 +508,19 @@ static void dump_batman_bcast(unsigned char *packet_buff, ssize_t buff_len, int
 	       get_name_by_macaddr((struct ether_addr *)bcast_packet->orig, read_opt),
 	       ntohl(bcast_packet->seqno));
 
-	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct bcast_packet),
-		      buff_len - ETH_HLEN - sizeof(struct bcast_packet),
+	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_bcast_packet),
+		      buff_len - ETH_HLEN - sizeof(struct batadv_bcast_packet),
 		      read_opt, time_printed);
 }
 
 static void dump_batman_frag(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct unicast_frag_packet *unicast_frag_packet;
+	struct batadv_unicast_frag_packet *unicast_frag_packet;
 
-	LEN_CHECK((size_t)buff_len - ETH_HLEN, sizeof(struct unicast_frag_packet), "BAT FRAG");
-	LEN_CHECK((size_t)buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet), (size_t)ETH_HLEN, "BAT FRAG (unpacked)");
+	LEN_CHECK((size_t)buff_len - ETH_HLEN, sizeof(struct batadv_unicast_frag_packet), "BAT FRAG");
+	LEN_CHECK((size_t)buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet), (size_t)ETH_HLEN, "BAT FRAG (unpacked)");
 
-	unicast_frag_packet = (struct unicast_frag_packet *)(packet_buff + ETH_HLEN);
+	unicast_frag_packet = (struct batadv_unicast_frag_packet *)(packet_buff + ETH_HLEN);
 
 	if (!time_printed)
 		time_printed = print_time();
@@ -456,20 +531,48 @@ static void dump_batman_frag(unsigned char *packet_buff, ssize_t buff_len, int r
 	printf("%s: FRAG, seq %hu, ttvn %d, ttl %hhu, flags [%c%c], ",
 	       get_name_by_macaddr((struct ether_addr *)unicast_frag_packet->dest, read_opt),
 	       ntohs(unicast_frag_packet->seqno), unicast_frag_packet->ttvn, unicast_frag_packet->header.ttl,
-	       (unicast_frag_packet->flags & UNI_FRAG_HEAD ? 'H' : '.'),
-	       (unicast_frag_packet->flags & UNI_FRAG_LARGETAIL ? 'L' : '.'));
+	       (unicast_frag_packet->flags & BATADV_UNI_FRAG_HEAD ? 'H' : '.'),
+	       (unicast_frag_packet->flags & BATADV_UNI_FRAG_LARGETAIL ? 'L' : '.'));
 
-	if (unicast_frag_packet->flags & UNI_FRAG_HEAD)
-		parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct unicast_frag_packet),
-			      buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet),
+	if (unicast_frag_packet->flags & BATADV_UNI_FRAG_HEAD)
+		parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_frag_packet),
+			      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet),
 			      read_opt, time_printed);
 	else
-		printf("length %zu\n", (size_t)buff_len - ETH_HLEN - sizeof(struct unicast_frag_packet));
+		printf("length %zu\n", (size_t)buff_len - ETH_HLEN - sizeof(struct batadv_unicast_frag_packet));
+}
+
+static void dump_batman_4addr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
+{
+	struct ether_header *ether_header;
+	struct batadv_unicast_4addr_packet *unicast_4addr_packet;
+
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header), sizeof(struct batadv_unicast_4addr_packet), "BAT 4ADDR");
+	LEN_CHECK((size_t)buff_len - sizeof(struct ether_header) - sizeof(struct batadv_unicast_4addr_packet),
+		sizeof(struct ether_header), "BAT 4ADDR (unpacked)");
+
+	ether_header = (struct ether_header *)packet_buff;
+	unicast_4addr_packet = (struct batadv_unicast_4addr_packet *)(packet_buff + sizeof(struct ether_header));
+
+	if (!time_printed)
+		time_printed = print_time();
+
+	printf("BAT %s > ",
+	       get_name_by_macaddr((struct ether_addr *)ether_header->ether_shost, read_opt));
+
+	printf("%s: 4ADDR, subtybe %hhu, ttvn %d, ttl %hhu, ",
+	       get_name_by_macaddr((struct ether_addr *)unicast_4addr_packet->u.dest, read_opt),
+	       unicast_4addr_packet->subtype, unicast_4addr_packet->u.ttvn,
+	       unicast_4addr_packet->u.header.ttl);
+
+	parse_eth_hdr(packet_buff + ETH_HLEN + sizeof(struct batadv_unicast_4addr_packet),
+		      buff_len - ETH_HLEN - sizeof(struct batadv_unicast_4addr_packet),
+		      read_opt, time_printed);
 }
 
 static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read_opt, int time_printed)
 {
-	struct batman_ogm_packet *batman_ogm_packet;
+	struct batadv_ogm_packet *batman_ogm_packet;
 	struct ether_header *eth_hdr;
 
 	eth_hdr = (struct ether_header *)packet_buff;
@@ -477,7 +580,8 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 	switch (ntohs(eth_hdr->ether_type)) {
 	case ETH_P_ARP:
 		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
-			dump_arp(packet_buff + ETH_HLEN, buff_len - ETH_HLEN, time_printed);
+			dump_arp(packet_buff + ETH_HLEN, buff_len - ETH_HLEN,
+				 eth_hdr, read_opt, time_printed);
 		break;
 	case ETH_P_IP:
 		if ((dump_level & DUMP_TYPE_NONBAT) || (time_printed))
@@ -488,50 +592,58 @@ static void parse_eth_hdr(unsigned char *packet_buff, ssize_t buff_len, int read
 			dump_vlan(packet_buff, buff_len, read_opt, time_printed);
 		break;
 	case ETH_P_BATMAN:
-		batman_ogm_packet = (struct batman_ogm_packet *)(packet_buff + ETH_HLEN);
+		batman_ogm_packet = (struct batadv_ogm_packet *)(packet_buff + ETH_HLEN);
+
+		if ((read_opt & COMPAT_FILTER) &&
+		    (batman_ogm_packet->header.version != BATADV_COMPAT_VERSION))
+			return;
 
 		switch (batman_ogm_packet->header.packet_type) {
-		case BAT_OGM:
+		case BATADV_IV_OGM:
 			if (dump_level & DUMP_TYPE_BATOGM)
-				dump_batman_ogm(packet_buff, buff_len, read_opt, time_printed);
+				dump_batman_iv_ogm(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_ICMP:
+		case BATADV_ICMP:
 			if (dump_level & DUMP_TYPE_BATICMP)
 				dump_batman_icmp(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_UNICAST:
+		case BATADV_UNICAST:
 			if (dump_level & DUMP_TYPE_BATUCAST)
 				dump_batman_ucast(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_BCAST:
+		case BATADV_BCAST:
 			if (dump_level & DUMP_TYPE_BATBCAST)
 				dump_batman_bcast(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_VIS:
+		case BATADV_VIS:
 			if (dump_level & DUMP_TYPE_BATVIS)
-				printf("Warning - batman vis packet received: function not implemented yet\n");
+				fprintf(stderr, "Warning - batman vis packet received: function not implemented yet\n");
 			break;
-		case BAT_UNICAST_FRAG:
+		case BATADV_UNICAST_FRAG:
 			if (dump_level & DUMP_TYPE_BATFRAG)
 				dump_batman_frag(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_TT_QUERY:
+		case BATADV_TT_QUERY:
 			if (dump_level & DUMP_TYPE_BATTT)
 				dump_batman_tt(packet_buff, buff_len, read_opt, time_printed);
 			break;
-		case BAT_ROAM_ADV:
+		case BATADV_ROAM_ADV:
 			if (dump_level & DUMP_TYPE_BATTT)
 				dump_batman_roam(packet_buff, buff_len, read_opt, time_printed);
 			break;
+		case BATADV_UNICAST_4ADDR:
+			if (dump_level & DUMP_TYPE_BATUCAST)
+				dump_batman_4addr(packet_buff, buff_len, read_opt, time_printed);
+			break;
 		default:
-			printf("Warning - packet contains unknown batman packet type: 0x%02x\n", batman_ogm_packet->header.packet_type);
+			fprintf(stderr, "Warning - packet contains unknown batman packet type: 0x%02x\n", batman_ogm_packet->header.packet_type);
 			break;
 		}
 
 		break;
 
 	default:
-		printf("Warning - packet contains unknown ether type: 0x%04x\n", ntohs(eth_hdr->ether_type));
+		fprintf(stderr, "Warning - packet contains unknown ether type: 0x%04x\n", ntohs(eth_hdr->ether_type));
 		break;
 	}
 }
@@ -638,8 +750,12 @@ int tcpdump(int argc, char **argv)
 
 	dump_level = dump_level_all;
 
-	while ((optchar = getopt(argc, argv, "hnp:x:")) != -1) {
+	while ((optchar = getopt(argc, argv, "chnp:x:")) != -1) {
 		switch (optchar) {
+		case 'c':
+			read_opt |= COMPAT_FILTER;
+			found_args += 1;
+			break;
 		case 'h':
 			tcpdump_usage();
 			return EXIT_SUCCESS;
@@ -666,7 +782,7 @@ int tcpdump(int argc, char **argv)
 	}
 
 	if (argc <= found_args) {
-		printf("Error - target interface not specified\n");
+		fprintf(stderr, "Error - target interface not specified\n");
 		tcpdump_usage();
 		return EXIT_FAILURE;
 	}
@@ -686,14 +802,14 @@ int tcpdump(int argc, char **argv)
 		dump_if->dev = argv[found_args];
 
 		if (strlen(dump_if->dev) > IFNAMSIZ - 1) {
-			printf("Error - interface name too long: %s\n", dump_if->dev);
+			fprintf(stderr, "Error - interface name too long: %s\n", dump_if->dev);
 			goto out;
 		}
 
 		dump_if->raw_sock = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
 
 		if (dump_if->raw_sock < 0) {
-			printf("Error - can't create raw socket: %s\n", strerror(errno));
+			fprintf(stderr, "Error - can't create raw socket: %s\n", strerror(errno));
 			goto out;
 		}
 
@@ -702,7 +818,7 @@ int tcpdump(int argc, char **argv)
 
 		res = ioctl(dump_if->raw_sock, SIOCGIFHWADDR, &req);
 		if (res < 0) {
-			printf("Error - can't create raw socket (SIOCGIFHWADDR): %s\n", strerror(errno));
+			fprintf(stderr, "Error - can't create raw socket (SIOCGIFHWADDR): %s\n", strerror(errno));
 			close(dump_if->raw_sock);
 			goto out;
 		}
@@ -715,7 +831,7 @@ int tcpdump(int argc, char **argv)
 		case ARPHRD_IEEE80211_RADIOTAP:
 			break;
 		default:
-			printf("Error - interface '%s' is of unknown type: %i\n", dump_if->dev, dump_if->hw_type);
+			fprintf(stderr, "Error - interface '%s' is of unknown type: %i\n", dump_if->dev, dump_if->hw_type);
 			goto out;
 		}
 
@@ -725,7 +841,7 @@ int tcpdump(int argc, char **argv)
 		res = ioctl(dump_if->raw_sock, SIOCGIFINDEX, &req);
 
 		if (res < 0) {
-			printf("Error - can't create raw socket (SIOCGIFINDEX): %s\n", strerror(errno));
+			fprintf(stderr, "Error - can't create raw socket (SIOCGIFINDEX): %s\n", strerror(errno));
 			close(dump_if->raw_sock);
 			goto out;
 		}
@@ -737,7 +853,7 @@ int tcpdump(int argc, char **argv)
 		res = bind(dump_if->raw_sock, (struct sockaddr *)&dump_if->addr, sizeof(struct sockaddr_ll));
 
 		if (res < 0) {
-			printf("Error - can't bind raw socket: %s\n", strerror(errno));
+			fprintf(stderr, "Error - can't bind raw socket: %s\n", strerror(errno));
 			close(dump_if->raw_sock);
 			goto out;
 		}
@@ -763,7 +879,7 @@ int tcpdump(int argc, char **argv)
 			continue;
 
 		if (res < 0) {
-			printf("Error - can't select on raw socket: %s\n", strerror(errno));
+			fprintf(stderr, "Error - can't select on raw socket: %s\n", strerror(errno));
 			continue;
 		}
 
@@ -774,12 +890,12 @@ int tcpdump(int argc, char **argv)
 			read_len = read(dump_if->raw_sock, packet_buff, sizeof(packet_buff));
 
 			if (read_len < 0) {
-				printf("Error - can't read from interface '%s': %s\n", dump_if->dev, strerror(errno));
+				fprintf(stderr, "Error - can't read from interface '%s': %s\n", dump_if->dev, strerror(errno));
 				continue;
 			}
 
 			if ((size_t)read_len < sizeof(struct ether_header)) {
-				printf("Warning - dropping received packet as it is smaller than expected (%zu): %zd\n",
+				fprintf(stderr, "Warning - dropping received packet as it is smaller than expected (%zu): %zd\n",
 					sizeof(struct ether_header), read_len);
 				continue;
 			}
